#!/usr/bin/perl

my $version = "2.14";

# Programmed by:
# Marco van Berkum, m.v.berkum@obit.nl
# Kristian Vlaardingerbroek, kris@obit.nl
# Raymond Vrolijk, raymond.vrolijk@veronica.nl
# Pepijn Vissers, zoef@zoefdehaas.nl
# Herman Poortermans, herman@ofzo.nl
#
# Website: http://ws.obit.nl
#
# Hackbot is also a freshmeat project
# http://freshmeat.net/projects/hackbot/


use IO::Socket;
use Net::hostent;
use Getopt::Std;

# core functionality

&getopts('AitfmsSdrw:z:a:XF:M:O:P:');

use vars qw($opt_A $opt_i $opt_t $opt_f $opt_m $opt_s $opt_S $opt_d $opt_r $opt_w $opt_z $opt_a $opt_X $opt_F $opt_O $opt_P);

if (!$opt_F) { $host = $ARGV[0]; } else {chomp $opt_F; $targetfile = $opt_F};

if( ( !$host && !$opt_F ) || ( $host && $opt_F ) || (!$opt_A && !$opt_i && !$opt_t && !$opt_f && !$opt_m && !$opt_s && !$opt_S && !$opt_d && !$opt_r && !$opt_w && !$opt_z && !$opt_a && !$opt_X && !$opt_F && !$opt_O && !$opt_P) ) {
        print("Hackbot version $version\nusage: ./hackbot.pl <options> <ip,hostname,net,or range>\n\n");
        print("or: ./hackbot.pl <options> -F <targetfile>\n\n");
        print("Available options:\n");
	print("-O <output file>\n");
        print("-A\t\tAll scan types\n");
	print("-i\t\tidentd scan\n");
	print("-t\t\tSimple Telnet Fingerprint\n");
        print("-f\t\tFTP scan\n");
        print("-m\t\tMTA scan\n");
        print("-s\t\tSSH scan\n");
	print("-S\t\tSpammer check\n");
        print("-d\t\tDNS scan\n");
	print("-r\t\tRegistrar whois lookup\n");
	print("-P <host:port>\tUse proxy for scanning\n");
        print("-w <type>\tWebserver scan\n");
        print("\t\ta  All scans\n");
        print("\t\tv  Version scan\n");
        print("\t\to  HTTP Options scan\n");
        print("\t\tc  CGI scan\n");
        print("\t\ti  IDA scan\n");
        print("\t\tu  Unicode scan\n");
        print("\t\tn  Nimda scan\n");
	print("-z <NN>\t\tSet time in seconds between each CGI poll \(to be less conspicuous\)\n");
	print("-a <NN>\t\tSet alternative webserver port\n");
	print("-X\t\tX allow check\n\n");
        print("examples:\t./hackbot.pl -f -m -w oci localhost\n");
        print("\t\t./hackbot.pl -A 192.168.75.3 195.0.0.255\n");
	print("\t\t./hackbot.pl -s 192.168.75.0\/26\n\n");
	exit(0);
}

$database = "/usr/local/etc/cgi.db";
$fingerprintdb = "/usr/local/etc/fingerprint.db";

# print(`clear`);
print("\n");
&banner;

$noprint = 1; log_print(`date`); log_print("\n"); undef($noprint);

if($opt_w) {
        if(($opt_w !~ /^[aociunv]+$/ ))
        {
                print "Unknown webserver scan type: " . $opt_w . "\n";
                exit(0);
        }
}



if($opt_a) {
	if(!$opt_A && !$opt_w) {
		print("Please define more options\n");
		print("Quitting...\n");
		unlink "$host.output";
		exit(0);
		}
	chomp($opt_a);
	$port = $opt_a;
} else {
	$port = "80";
}


if($opt_z) {
	if(!$opt_A && !$opt_w) {
		print("Please define more options\n");
		print("Quitting...\n");
		unlink "$host.output";
		exit(0);
		}
	chomp($opt_z);
}

if($opt_P) {
	($proxy, $proxyport) = split(/:/, $opt_P);
	chomp($proxy);
	chomp($proxyport);
	$userinputproxythingy = $proxy;
	if ($proxy =~ /[A-z]/) {
		log_print("Trying to resolve proxyhost $proxy\n");
		my $handler = gethost($proxy);
		if(!$handler) {
			log_print("proxyhost $proxy does not resolve, will continue scan without using a proxy\n\n");
			undef($proxy);
			undef($proxyport);
		} else {
		$proxy = inet_ntoa(@{$handler->addr_list}[0]);
		log_print("Resolved $userinputproxythingy to $proxy\n");
		log_print("Testing if the proxy works..\n\n");
		&proxytest($proxy, $host);
		$tested = 1;
		}
	}
	if ($proxy !~ /[A-z]/) {
		log_print("Testing if the proxy works..\n") unless($tested);
		&proxytest($proxy, $host) unless($tested);
	}

	undef($tested);
}


if($opt_F) {
        $date = `date`;
        open(TARGETLIST,"$targetfile") || die "$0:     Could not read from $targetfile! ($!)";
        while (<TARGETLIST>) { chomp; push(@targetlist, $_); }

        foreach $host (@targetlist) {
        log_print("Checking $host ...\n\n");
        &scanit }
	} else {
		if ($host=~/[A-z]/)
		{
			log_print("Checking named host $host ...");
			&scanit;
		} else { 
        	   &GetRange;
		   unless("$start" eq "$end") { log_print("Range Scan from $start to $end\n"); }	
		   foreach $host (@targetlist) {
       		   	log_print("\nChecking $host ...\n\n");
        		&scanit; 
			}		
     }
}

&endbanner;

# end core


###############
# Subroutines #
###############

sub scanit {

        if (!&check_ip($host)) {
                log_print(" trying to resolve hostname $host\n");
                my $handler = gethost($host);
                if(!$handler) {
                        log_print("$host does not resolve, I died\n\n");
                        if($opt_O) {
                          unlink $opt_O;
                        } else {
                          unlink "$host.output";
                        }
                        exit(0);
                        }
                $target = inet_ntoa(@{$handler->addr_list}[0]);
                log_print("resolved host to: $target\n\n");
                $host_resolves = 1;
                } else {
                $target = $host;
                $host_resolves = 0;
                }
	if($opt_A){
	$opt_i="1";$opt_t="1";
	$opt_f="1";$opt_m="1";
	$opt_s="1";$opt_S="1";
        $opt_d="1";$opt_r="1";
	$opt_w="a";$opt_X="1";
	#now you can undef opt_A in case of no web. 
	# undef after this line
	}

	if($opt_S) {
		&spamcheck;
	}
	if($opt_i) {
		&ident;
	}
	if($opt_t) {
		&telnetfprint;
	}
        if($opt_f) {
                &ftp;
        }
        if($opt_m) {
                &mta;
        }
        if($opt_s) {
                &ssh;
        }
        if($opt_d) {
                &dns;
        }
	if($opt_r) {
		&whois;
	}
	if($opt_X) {
		&xcheck;
	}
        if($opt_w) {
                &checkweb;

                if(($opt_w =~ /o/)||($opt_w =~ /a/)) {
                        &http_options unless $noweb;
                }
                if(($opt_w =~ /c/)||($opt_w =~ /a/)) {
                        &cgi unless $noweb;
                        &netscape unless $noweb;
                }
                if(($opt_w =~ /i/)||($opt_w =~ /a/)) {
                        &ida unless $noweb;
                }
                if(($opt_w =~ /u/)||($opt_w =~ /a/)) {
                        &unicode unless $noweb;
                        &unicode2 unless $noweb;
                }
                if(($opt_w =~ /n/)||($opt_w =~ /a/)) {
                        &nimda unless $noweb;
                }
        } 

}


sub check_ip {
	my $ipaddr = shift;

	if ($ipaddr !~ /^[0-9\.]+$/) { return 0 }
	if ($ipaddr !~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) { return 0 }
	for ($1, $2, $3, $4) { return 0 if ($_ > 255) }
	return 1;
}

sub banner {
	log_print("###############################################################\n");
	log_print("#      HackBot v$version 2002 / http://ws.obit.nl/hackbot/        #\n");
	log_print("#                                                             #\n");
	log_print("# Marco van Berkum            - m.v.berkum\@obit.nl            #\n");	
	log_print("# Kristian Vlaardingerbroek   - kris\@obit.nl                  #\n");
	log_print("# Raymond Vrolijk             - raymond.vrolijk\@veronica.nl   #\n");
	log_print("# Pepijn Vissers              - zoef\@zoefdehaas.nl            #\n");
	log_print("# Martijn Mooijman            - foobar\@obit.nl                #\n");
	log_print("# Herman Poortermans          - herman\@ofzo.nl                #\n");
	log_print("###############################################################\n");
	log_print("\n");
}


sub cgi {
	if (!-e $database) { 
		$nodb = 1;
		}

	open CGI, "<$database" unless($nodb);
	# potential bugfix
	undef($exploit); undef($description); undef($advisory);

	# actual code
	log_print("Checking the webserver on port $port for various potential problems\n") unless($nodb);
	log_print("------------------------------------------------------------------\n") unless($nodb);

	while(1) {
        	if($nodb) { 
			log_print("No db found it should be in $database, skipping CGI scan\n");
			last;
		}	

		if($proxy) {
		print("Scanning via proxy $proxy\n\n");
                while(<CGI>) {
			if($opt_z) { sleep($opt_z); }
                        ($exploit, $description, $advisory) = split(/\!/, $_);
                        chomp($exploit, $description, $advisory);
                        my $sock = new IO::Socket::INET (PeerAddr => $proxy, PeerPort => $proxyport, Proto => 'tcp', Timeout => '5');
                        if($sock && $exploit) {
                                if($host_resolves = 1) {
                                        print $sock "GET http://$host$exploit HTTP/1.0\r\n\r\n";
                                } else {
                                        print $sock "GET http://$target$exploit HTTP/1.0\r\n\r\n";
                                }
                                sysread($sock, $buff, 15000);
                                if($buff =~ /200 OK/) {
                                log_print("* $exploit found!\n");
                                log_print("* $description\n") unless (!$description);
                                log_print("* $advisory\n") unless (!$advisory);
                                log_print("\n");
                                close($sock); } 
                        	} 
		}} else
			{
		while(<CGI>) {
			if($opt_z) { sleep($opt_z); }
			($exploit, $description, $advisory) = split(/\!/, $_);
			chomp($exploit, $description, $advisory);
			my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
			if($sock) {
				if($host_resolves = 1) {
					print $sock "GET http://$host$exploit HTTP/1.0\r\n\r\n";
				} else {
					print $sock "GET $exploit HTTP/1.0\r\n\r\n";
				}
				sysread($sock, $buff, 15000);
				if($buff =~ /200 OK/) {
				log_print("* $exploit found!\n");
				log_print("* $description\n") unless (!$description);
				log_print("* $advisory\n") unless (!$advisory);
				log_print("\n");
				close($sock);
				}
			}	
		} 
	}

	close CGI;
	last;
 	}	
}

sub mta {
	log_print("Trying MTA - Relaying, VRFY and EXPN\n");
	log_print("------------------------------------\n");
	my $sock = new IO::Socket::INET (
        	PeerAddr => $target,
        	PeerPort => '25',
        	Proto => 'tcp',
        	Timeout => '7'
        	); 

	if(!$sock) {
		log_print("No MTA\n\n");
	}

	while($sock) {
		$cnt = sysread($sock, $buff, 15000);
		if(!$cnt) {
			log_print("Hmmm weird socket error, I died\n\n");
			close($sock);
			last;
		}		
		log_print($buff); 

                if($buff =~ /^220 \*\*\*\*\*\*\*\*\*/) {
		log_print("This mailserver is probably protected by a Cisco Pix Firewall with MailGuard enabled\n");
		log_print("\nLets try to outsmart it \:\)\_\n\n");
		print $sock "data\r\n";
		sleep(3);
		sysread($sock, $buff, 1500);
		print("return after DATA command is: $buff\n");

		if($buff !~ /^500/) {
		log_print("Sending EHLO command\n");
		print $sock "EHLO hackbot\r\n";
		sleep(3);
		sysread($sock, $buff, 1500);

		if($buff !~ /^500/) {
		log_print("worked..\n\n");
		log_print("$buff\n");
		} else { 
		log_print("didn't work..\n\n");
		}
						
		log_print("Sending HELP command\n");
		print $sock "HELP\r\n";
		sleep(3);
		sysread($sock, $buff, 1500);

		if($buff !~ /^500/) {
		log_print("worked..\n\n");
		log_print("$buff\n");
		} else {
		log_print("didn't work..\n\n");
		}
		
		log_print("Trying VRFY command\n");
		print $sock "vrfy root\r\n";
		sleep(3);
		sysread($sock, $buff, 1500);

		if ($buff =~ /^250/) {
		log_print("worked..\n\n");
		log_print("$buff\n");
		} else { 
		log_print("didn't work..\n\n");
		}

		log_print("Trying EXPN command\n");
		print $sock "expn root\r\n";
		sleep(3);
		sysread($sock, $buff, 1500);

		if ($buff =~ /^250/) {
		log_print("worked..\n\n");
		log_print("$buff\n");
		} else {
		log_print("didn't work..\n\n");
		}
		log_print("Done playing with PIX for now, let's continue with the normal commands\n\n");
		print $sock "RSET\r\n";
		}
		}

		undef($buff);
		sleep(1);
		print $sock "helo bla\r\n";
		sleep(2);
		sysread($sock, $buff, 150);
		print $sock "mail from: bill.gates\@microsoft.com\r\n";
		sysread($sock, $buff, 150);
		print $sock "rcpt to: bill.gates\@microsoft.com\r\n";
		sleep(2);
		sysread($sock, $buff, 150);

		if ($buff =~ /^250/) {
			log_print("Relaying allowed\n");
			} else {
			log_print("Relaying not allowed\n");
		}

		print $sock "vrfy root\r\n";
		sysread($sock, $buff, 15000);

		if ($buff =~ /^250/) {
			log_print("VRFY enabled\n");
			log_print("$buff\n");
			} else {
			log_print("VRFY not enabled\n");
		}

		print $sock "expn root\r\n";
		sysread($sock, $buff, 15000);

		if ($buff =~ /^250/) {
			log_print("EXPN enabled\n");
			print "$buff\n"
		} else {
		log_print("EXPN not enabled\n");
		}

		log_print("\n");
		close(!$sock);
		last;
		}
}

sub checkweb {
	undef($noweb);
        log_print("Checking for webserver on port $port\n");
	log_print("-----------------------------------\n");
        my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
        if ($sock) {
                if($host_resolves = 1) {
		        print $sock "HEAD http://$host/ HTTP/1.0\r\n\r\n";
                } else {
		        print $sock "HEAD / HTTP/1.0\r\n\r\n";
                }
		sleep(2);
		sysread($sock, $httpversion, 15000);

		if($opt_w =~ /v/ || $opt_w =~ /a/ || $opt_A) {
       		 	log_print("$httpversion\n");			

			if ($httpversion =~ /Apache/) {
				&modphpssl($httpversion);
			}

		} else {
		log_print("Webserver found\n\n");
		}
        } else {
        $noweb = 1;
        log_print("No webserver\n");
   }
}

sub http_options {
	log_print("HTTP options\n");
	log_print("------------\n"); 
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');

	# potential bugfix
	undef($allow); undef($public);

	# actual code

	if ($sock) {
                if($host_resolves = 1) {
			print $sock "OPTIONS http://$host/ HTTP/1.0\r\n\r\n";
		} else {
			print $sock "OPTIONS / HTTP/1.0\r\n\r\n";
		}
		sysread($sock, $buff, 5000);
		$allow = $buff; $public = $buff;
		($tmp, $allow) = split(/Allow:(.*)/, $buff);
		($tmp, $public) = split(/Public:(.*)/, $buff);
		log_print("Allow options : $allow\n") unless(!$allow);
		log_print("Public options: $public\n") unless(!$public);
		log_print("\n\n");
		close($sock);
	} 
		if ($httpversion =~ /IIS/) {
			&propfind($target);
			&isapi($target);
		}
}


sub ida {
@idas = ('ida','idq','idc');
log_print("\nChecking for ida\n");
log_print("----------------\n");
foreach(@idas) {
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
	if ($sock) {
		if($host_resolves = 1) {
			print $sock "GET http://$host/blabla.$_ HTTP/1.0\r\n\r\n";
		} else {
			print $sock "GET /blabla.$_ HTTP/1.0\r\n\r\n";
		}
		sleep(3);
		sysread($sock, $buff, 1000);
		if ($buff =~ /\:\\/) {
			($temp, $lastline) = split(/file\ /,$buff);
			($path) = split(/\ /,$lastline);
			log_print("$path\n");
 		}
		close($sock);
   	} 
}
log_print("\n");
}


sub unicode {
	@vuln = ('msadc','scripts');
	log_print("Unicode tests\n");
	log_print("-------------\n");
	log_print("Checking for unicode with ..\%c0\%af..\n");

	foreach(@vuln) {
		my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
		if ($sock) { $buff = ""; $current = $_;
			if($host_resolves = 1) {
				print $sock "GET http://$host/$current/..\%c0\%af../..\%c0\%af../..\%c0\%af../..\%c0\%af../..\%c0\%af../winnt/system32/cmd.exe\?\/c+bla HTTP/1.0\n\n";
			} else {
				print $sock "GET /$current/..\%c0\%af../..\%c0\%af../..\%c0\%af../..\%c0\%af../..\%c0\%af../winnt/system32/cmd.exe\?\/c+bla HTTP/1.0\n\n";
			}
			sysread($sock, $buff, 9000);
			close($sock);
			if ($buff =~ /Content-Type: application\/octet-stream/) {
				log_print("Unicode vulnerable with $current and ..\%c0\%af.. \n");
  				}
			close($sock);
  			}
 		}
		undef($current);
}


sub unicode2 {
	log_print("Checking for unicode with ..\%255c..\n");

	foreach(@vuln) {
		my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
		if ($sock) { $buff = ""; $current = $_;
			if($host_resolves = 1) {
				print $sock "GET http://$host/$current/..\%255c../..\%255c../..\%255c../..\%255c../..\%255c../winnt/system32/cmd.exe\?\/c+bla HTTP/1.0\n\n";
			} else {
				print $sock "GET /$current/..\%255c../..\%255c../..\%255c../..\%255c../..\%255c../winnt/system32/cmd.exe\?\/c+bla HTTP/1.0\n\n";
			}
			sysread($sock, $buff, 9000);
			close($sock);
				if ($buff =~ /Content-Type: application\/octet-stream/) {
					log_print("Unicode vulnerable with $current and ..\%255c.. \n");
  				}
			close($sock);
  			}
 		}
}

sub nimda {
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
	if ($sock) {
		log_print("\nchecking for nimda infection\n");
		log_print("----------------------------\n");
		if($host_resolves = 1) {
			print $sock "GET http://$host/blaat HTTP/1.0\n\r\n\r";
		} else {
			print $sock "GET /blaat HTTP/1.0\n\r\n\r";
		}
		sleep(5);
		sysread($sock, $buff, 15000);
			if ($buff =~ /eml/ig) {
			log_print("eml found at $target, it's probably infected\n");
     			}
		 close($sock);
   		}
}


sub dns { 
	log_print("Checking for DNS\n");
	log_print("----------------\n");
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 53, Proto => 'tcp', Timeout => '5');
	if ($sock) {                  
		$nsversion = `dig \@$target version.bind chaos txt`;
		($temp, $bind) = split(/\"/, $nsversion);
		$temp = "";
		log_print("Bindversion: $bind\n\n");
  	} else {
          	log_print("No DNS\n\n");
        }
    	close($sock);
}
 

sub ssh {
	log_print("Checking for SSH\n");
	log_print("----------------\n");

	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 22, Proto => 'tcp', Timeout => '5');

	if(!$sock) {
		log_print("No SSH\n\n");
	}
	while($sock) {
		sleep(5);
		$cnt = sysread($sock, $buff, 10000);
		if(!$cnt) {
			log_print("huh, fscking TCP wrappers, I died\n\n");
			last;
		}
		log_print("$buff\n");

   		close($sock); 
   		last;
	}
}


sub netscape {
if ($httpversion =~ /Netscape/) {
	@netscape = ('/?wp-cs-dump','?PageServices');
	foreach(@netscape) {
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');

	if (!$sock) {
	log_print("Webserver died?\n\n");
	}
	while($sock) {
	if($host_resolves = 1) {
        	print $sock "GET http://$host/$_ HTTP/1.0\r\n\r\n";
	} else {
        	print $sock "GET /$_ HTTP/1.0\r\n\r\n";
	}
        $cnt = sysread($sock, $buff, 15000);
	if (!$cnt) {
	log_print("Webserver died?\n\n");
	close($sock);
	last;
	}
	if ($buff =~ /200 OK/) {
	log_print("$_ found\!\n* Netscape $_ bug, try http://www.somehost.com/$_\n");
    }
   }
  }
 }
}

sub ftp {

log_print("\nChecking for FTP server\n");
log_print("-----------------------\n");
        my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 21, Proto => 'tcp', Timeout => '5');
        if (!$sock) {
        log_print("No FTP server\n\n");
        }
        while($sock) {
                print $sock "user anonymous\r\n";
                sleep(2);
                $cnt = sysread($sock, $buff, 10000);
                if(!$cnt) {
                log_print("huh, fscking TCP wrappers, I died\n\n");
                last;
                }
                log_print("FTP Server Found:\n$buff");
                print $sock "pass bla\@bla.com\r\n";
                sleep(2);
                $cnt = sysread($sock, $buff, 1000);
                if(!$cnt) {
                last;
                }
                if ($buff =~ /230/) {
                log_print("Anonymous login allowed\!\n\n");
                log_print("Checking for upload/ directory\n\n");
                print $sock "CWD upload\n";
                sleep(2);
                $cnt = sysread($sock, $buff, 1000);
                if(!$cnt){last;}
                if ($buff =~ /250/) {
                        log_print("Got it... checking write possibilities\n\n");
                        print $sock "STOR hackbotwashere.bla\r\n";
                        sleep(2);
                        $cnt = sysread($sock, $buff, 100);
                        if ($buff =~ /553/){
                                log_print("nope, no write-access\n\n");
                                } else {
                                log_print("yup... write-access\n\n");
                                } 
                       }
                } else {
                log_print("Anonymous login NOT allowed\n\n");
                }
                close($sock);
                last;
        }
}

sub GetRange {
	$start = $ARGV[0];
	$end = $ARGV[1];
	     if ($start=~/\*/){            
             @target=split/\./,$start;
             $start= ( &genipstart );  
             $end= ( &genipend);
             }
	($ippart, $bit) = split(/\//,$start);
		#CIDR
		if($bit ne "") 
		{ 
			($a,$b,$c,$d) = split(/\./,$ippart);

			if ($bit <= 23) 
			{
        				die "Error: CIDR Mask < /24 currently unsupported.\n";
   			 } elsif ($bit >= 31) {
       						 die "Error: CIDR Mask '$bit' is invalid.\n";
    					       }
    						$prefix = "$a.$b.$c.";
    						$network = $d;
    						$nhosts = (2 ** (32 - $bit));
    						$netmask = (256 - $nhosts);
   			 unless ($netmask > 0 && $netmask / $nhosts) 
    			{
       			 die "Invalid Network: Netmask $netmask, Hosts $nhosts\n" unless($netmask eq 0);
    			}
   			 $start=$ippart;	
			 $end="$prefix".($nhosts + $network  - 1);
		}

		if($start ne "" && $end eq "")
		{
		$end=$start;
		}
		
		if($start eq "" || $end eq "" ) { &InputError; };

	@IPstart = split(/\./,$start);
	@IPend = split(/\./,$end);
		&Check_Start_and_EndIP;

		#converts "short" (dotted) IPs to "long" (undotted) IPs

		$IPstart = ((@IPstart[0]*16777216)+(@IPstart[1]*65536)+(@IPstart[2]*256)+@IPstart[3]);
		$IPend = ((@IPend[0]*16777216)+(@IPend[1]*65536)+(@IPend[2]*256)+@IPend[3]);

		if($IPend < $IPstart) { die "Can't scan backwards"; }
	$CountIp = $IPstart;
        $EndIp = $IPend+1;

		while($CountIp ne $EndIp) 
		{
 				  @Class = &GetIP($CountIp);
                         	  push(@targetlist, "@Class[1]\.@Class[2]\.@Class[3]\.@Class[4]");
				  $CountIp++;
 		}
}

sub GetIP{
	 # converts "long" (undotted) IPs into "short" (dotted) IPs
	 my($ip) = @_;
 	 @Class[1] = int ( $ip / 256**3);
 	 @Class[2] = int ( $ip % 256**3 / 256**2 );
 	 @Class[3] = int ( $ip % 256**3 % 256**2 / 256**1 );
 	 @Class[4] = int ( $ip % 256**3 % 256**2 % 256**1 );
 	 @Class;
}

sub InputError {
 print "need [starting IP] [ending IP]\n\n";
 die "Wrong input";
}


sub Check_Start_and_EndIP{
                   if (!&check_ip($start)) {
                   print "\nStarting IP is not valid\n";
                   die "Exiting on bad starting IP";
                   }
  
                   if (!&check_ip($end)) {
                   print "\nEnding IP is not valid\n";
                   die "Exiting on bad ending IP";
		   }
}

sub genipstart { for(@target){ if( $_=~/\*/){ push(@tmp, 0);}
                             else {push(@tmp, $_);}
                               }
                "$tmp[0].$tmp[1].$tmp[2].$tmp[3]"
                }

sub genipend { for(@target){if( $_=~/\*/){ push(@tmp2, 255);}
                           else {push(@tmp2, $_);}
                             }
               "$tmp2[0].$tmp2[1].$tmp2[2].$tmp2[3]"
             }
             
sub log_print($) {
                my $logstring = shift;

                if($opt_O) {
                open LOGFILE, ">>$opt_O";
                print LOGFILE "$logstring";
                print("$logstring") unless($noprint);
                close LOGFILE;

                } else {

                open LOGFILE, ">>$host.output";
                print LOGFILE "$logstring";
                print("$logstring") unless($noprint);
                close LOGFILE;
  }
}

sub telnetfprint {

if($telnet ne "0") {

log_print("Trying simple Telnet fingerprint\n");
log_print("--------------------------------\n");


if (!-e $fingerprintdb) {
        log_print("HUH db not found, it should be in $fingerprintdb\n");
        log_print("Skipping Telnet fingerprint\n");
        $nofp = 1;
        }


my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 23, Proto => 'tcp', Timeout => '5') unless($nofp);
	undef($fingerprint); undef($description); undef($return); undef($submitter);

	if(!$sock) {
	log_print("No telnet\n") unless($nofp);
	}

        while($sock) {
	sleep(3);

	$cnt = sysread($sock, $buff, 14);
	if(!$cnt) {
	log_print("Hmmm, probably TCP wrappers\n") unless($nofp);
	last;
	}

	@rets = split(//, $buff);
	foreach $currentret (@rets){
	$fingerprint = $fingerprint . ord($currentret); }
	close($sock);
	log_print("Possible OS's\n\n");
	log_print("Fingerprint:\t\t\t$fingerprint\n") unless($nofp);
	last;
        }

if($fingerprint) {

        open FINGERPRINTDB, "<$fingerprintdb" || last("No fingerprint database, should be /usr/local/etc/fingerprint.db\n");

        while(<FINGERPRINTDB>) {
        ($description, $return, $submitter) = split(/\!/, $_);
        if("$fingerprint" eq "$return") {
        $hit = 1;
        log_print("Description:\t\t\t$description\n") unless($nofp);
        chomp($submitter);
        log_print("Fingerprint submission by:\t$submitter\n\n") unless($nofp);

        }

        }
        close FINGERPRINTDB;

        if(!$hit) {
        log_print("Fingerprint not found in database. If you know what it is please\nsubmit it to fingerprint\@obit.nl\n") unless($nofp);
	undef($nofp);
        }

     }
  }
}


sub ident
{

    # todo grammar checks. 
    # how does it work ?
    # well, we want to know which user is running what..
    # so we connect to some service lets say 80, www.
    # Then we connect to auth, and enter the target and source port of the web connection
    # it looks like 80,2003 now auth kindly returns a line wich contains the user that runs
    # the www service.
    # comments can be sent to herman@ofzo.nl
log_print("Identd\n");
log_print("------\n");
    my $sock = new IO::Socket::INET(
                                    PeerAddr => $target,
                                    PeerPort => '113',
                                    Proto    => 'tcp',
                                    Timeout  => '5'
                                    );

    if ($sock)
    {

        # hmmz we got signal, why not use the same socket for getting the user of ident 
        ($lp,undef) =
          unpack_sockaddr_in(getsockname($sock))
          ;    #get source port of local conection.
        print $sock "113,$lp\r\n";
        sysread($sock, $buff, 50);
        (undef, undef, undef, $undef) = split (/:/, $buff);
        log_print ("Port\tState\tService\tOwner\n");
        log_print ("----\t-----\t-------\t-----\n");
        log_print ("113\tOpen\tident\t$undef");
        close($sock);
        undef $lp;
        &full_identd_scan;
    } else
    {
        # skip rest of tests.. no identd no glory
        log_print("No Identd on $target, skipping full identd scan\n\n");
    }
};

sub full_identd_scan
{

    # full ident scan first setup target service. if it exists check ident for owner
    # hashed for easy maintanance.
    %ports_and_servicenames = (
                               21 => "ftp",
                               23 => "telnet",
                               25 => "mail",
                               80 => "www",
                               110 => "pop3"
                               ); #todo sorting order. so output will be incremental.
                                  # All your ports are belong to us!
    while (($identport, $service) = each(%ports_and_servicenames))
    # its geting hard to come up with usefull variable names these days
    {
        my $sock = new IO::Socket::INET(
                                        PeerAddr => $target,
                                        PeerPort => $identport,
                                        Proto    => 'tcp',
                                        Timeout  => '5'
                                        );

	if(!$sock and $identport eq "23") { $telnet = "0"; }
        if ($sock)
        {

            #And now for something completely similar.
            # setup sock2
	    my $sock2 =
              new IO::Socket::INET(
                                   PeerAddr => $target,
                                   PeerPort => '113',
                                   Proto    => 'tcp',
                                   Timeout  => '5'
                                   );
            ($lp,undef) = unpack_sockaddr_in(getsockname($sock));
            print $sock2 "$identport,$lp\r\n";
            sysread($sock2, $buff2, 50);
            if ($buff2 =~ /ERROR/)
            {
                log_print ("$identport\tError\t$service\tError\n");
            } else
            {   #undef defined!
                (undef, undef, undef, $undef) =
                  split (/:/, $buff2);   # third entry is systype, 99,9% its UNIX)
                log_print ("$identport\tOpen\t$service\t$undef"); 
                undef $buff;undef $buff2;undef $lp;undef $undef;
		close($sock);#undef undef undef... and undef.       
                close($sock2);#today is definetly my undef day. #todo code cleanup ;)
            }
        } else
        {
            log_print ("$identport\tClosed\t$service\t---\n");
        }
    }
log_print("\n");
}

sub whois {

	@RESERVEDA	= ('0','1','2','5','7','10','23','27','31','36','37','39','41','42','58','59','60',
			   '69','70','71','72','73','74','75','76','77','78','79','82','83','84',
			   '85','86','87','88','89','90','91','92','93','94','95','96','97','98',
			   '99','100','101','102','103','104','105','106','107','108','109','110',
			   '111','112','113','114','115','116','117','118','119','120','121','122',
			   '123','124','125','126','127','197','221','222','223','224','225','226',
			   '227','228','229','230','231','232','233','234','235','236','237','238','239',
			   '240','241','242','243','244','245','246','247','248','249','250','251','252',
			   '253','254','255');
	@RESERVEDB	= ('172.16','172.17','172.18','172.19','172.20','172.21','172.22','172.23',
			   '172.24','172.25','172.26','172.27','172.28','172.29','172.30','172.31',
			   '192.168','169.254');
	@RIPE		= ('62','80','81','193','194','195','212','213','217');
	@ARIN		= ('24','63','64','65','66','67','68','199','200','204','205','206','207',
			   '208','209','216');
	@APNIC		= ('61','202','203','210','211','218','219','220');

# determine range to be RESERVED, RIPE, ARIN or APNIC
($firstoct, $secoct) = split(/\./, $target);

if($ipmap = grep(/$firstoct!/, @RESERVEDA) or $ipmap = grep(/$firstoct\.$secoct/, @RESERVEDB))
	{ 
	log_print "reserved by IANA\n"; 
	$IANA = 1; 
	} elsif ($ipmap = grep(/$firstoct/, @RIPE)) {
	log_print("RIPE registrar\n");
	$RIPE = 1;
	} elsif ($ipmap = grep(/$firstoct/, @ARIN)) {
	log_print("ARIN registrar\n");
	$ARIN = 1;
	} elsif ($ipmap = grep(/$firstoct/, @APNIC)) {
	log_print("APNIC registrar\n");
	$APNIC = 1;
	} else {
	log_print("Don't know where to search whois data, could be anywhere, canceling whois checkup\n\n");
	$IANA = 1;
	}

if(!$IANA) {
	if($RIPE) 
	{
	&ripe;
	} elsif($APNIC) {
	&apnic;
	} elsif($ARIN) {
	&arin;
	}
	undef($IANA);
	}

}


sub ripe {
log_print("Checking Whois info \@RIPE\n");
log_print("-------------------------\n");
	my $sock = new IO::Socket::INET (PeerAddr => 'whois.ripe.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
	if($sock) {
		print $sock "$target\n";
		sleep(5);
		sysread($sock, $buff, 15000);
		close($sock);
	}

	($foobar, $inetnum)	= split(/inetnum\:\s/, $buff);	($inetnum, $foobar)	= split(/\n/, $inetnum);
	($foobar, $netname)	= split(/netname\:\s/, $buff);	($netname, $foobar)	= split(/\n/, $netname);
	($foobar, $descr)	= split(/descr\:\s/, $buff);	($descr, $foobar)	= split(/\n/, $descr);
	$descr =~ s/^\s{2}//;
	($foobar, $country)	= split(/country\:\s/, $buff);	($country, $foobar)	= split(/\n/, $country);
	($foobar, $adminc)	= split(/admin\-c\:\s/, $buff);	($adminc, $foobar)	= split(/\n/, $adminc);

if($adminc) {
	my $sock = new IO::Socket::INET (PeerAddr => 'whois.ripe.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
	if($sock) {
		print $sock "$adminc\n";
		sleep(5);
		sysread($sock, $buff, 1500);
		close($sock);
		}
}
		
	($foobar, $email)	= split(/e\-mail\:\s/, $buff);	($email, $foobar)	= split(/\n/, $email);
	$email =~ s/^\s//;

	log_print("Range:\t\t$inetnum\n");
	log_print("Netname:\t$netname\n");
	log_print("Description:\t$descr\n");
	log_print("Country:\t$country\n");
	log_print("Admin-C:\t$adminc\n");
	log_print("Email:\t\t$email\n\n") unless(!$email);

	undef($RIPE); undef($inetnum); undef($netname); undef($descr); 
	undef($country); undef($adminc); undef($email); undef($foobar);
}

sub arin {
print("Checking Whois info \@ARIN\n");
print("-------------------------\n");
 my $sock = new IO::Socket::INET (PeerAddr => 'whois.arin.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
        if($sock) {
                print $sock "$target\n";
                sleep(5);
                sysread($sock, $buff, 1500);
                close($sock);
        }
		($first)		= split(/\)/, $buff); ($tmp, $blk)	= split(/\(/, $first);

if($blk =~ /BLK/) {
log_print("Good! Got the netblock, investigating further...\n\n");
 my $sock = new IO::Socket::INET (PeerAddr => 'whois.arin.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
        if($sock) {
                print $sock "$blk\n";
                sleep(5);
                sysread($sock, $buff, 1500);
                close($sock);
        }
 $buff =~ s/The ARIN Registration Services Host contains ONLY Internet/\b/i;
 $buff =~ s/Network Information: Networks, ASN\'s, and related POC's\./\b/i;
 $buff =~ s/Please use the whois server at rs\.internic\.net for DOMAIN related/\b/i;
 $buff =~ s/Information and whois\.nic\.mil for NIPRNET Information\./\b/i;
 
 log_print("$buff\n");
 undef($blk); undef($blk); undef($ARIN); undef($buff); undef($first); undef($tmp);
 }
}

sub apnic {

log_print("Checking Whois info \@APNIC\n");
log_print("--------------------------\n");
	my $sock = new IO::Socket::INET (PeerAddr => 'whois.apnic.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
	if($sock) {
		print $sock "$target\n";
		sleep(5);
		sysread($sock, $buff, 15000);
		close($sock);
	}


	($foobar, $inetnum)	= split(/inetnum\:\s/, $buff);	($inetnum, $foobar)	= split(/\n/, $inetnum);
	($foobar, $netname)	= split(/netname\:\s/, $buff);	($netname, $foobar)	= split(/\n/, $netname);
	($foobar, $descr)	= split(/descr\:\s/, $buff);	($descr, $foobar)	= split(/\n/, $descr);
	$descr =~ s/^\s{2}//;
	($foobar, $country)	= split(/country\:\s/, $buff);	($country, $foobar)	= split(/\n/, $country);
	($foobar, $adminc)	= split(/admin\-c\:\s/, $buff);	($adminc, $foobar)	= split(/\n/, $adminc);

if($adminc) {
	my $sock = new IO::Socket::INET (PeerAddr => 'whois.apnic.net', PeerPort => 43, Proto => 'tcp', Timeout => '5');
	if($sock) {
		print $sock "$adminc\n";
		sleep(5);
		sysread($sock, $buff, 1500);
		close($sock);
		}
}

	($foobar, $email)	= split(/e\-mail\:\s/, $buff);	($email, $foobar)	= split(/\n/, $email);
	$email =~ s/^\s//;

	log_print("Range:\t\t$inetnum\n");
	log_print("Netname:\t$netname\n");
	log_print("Description:\t$descr\n");
	log_print("Country:\t$country\n");
	log_print("Admin-C:\t$adminc\n");
	log_print("Email:\t\t$email\n\n") unless(!$email);

	undef($APNIC); undef($inetnum); undef($netname); undef($descr);
	undef($country); undef($adminc); undef($email); undef($foobar);
}

sub modphpssl($) {
	my $versionstring = shift;

# bad apache check
	$badapachenotification = "Evil buggy Apache found!\nSeveral vulnerabilities, check http://www.apache.org\n\n";
	
        if($versionstring =~ /Apache\/1.3/) {
        ($temp1, $temp2) = split(/Apache\/1\.3\./, $versionstring);
	($temp2, $temp1) = split(/\s/, $temp2);
        if($temp2 lt "27") {
        log_print($badapachenotification);
        undef($temp1); undef($temp2);
        }
        }

        if($versionstring =~ /Apache\/1.2/) {
        log_print($badapachenotification);
        }

        if($versionstring =~ /Apache\/2.0./) {
        ($temp1, $temp2) = split(/Apache\/2\.0\./, $versionstring);
	($temp2, $temp1) = split(/\s/, $temp2);
        if($temp2 lt "43") {
        log_print($badapachenotification);
        undef($temp1); undef($temp2);
        }
        }
# end bad apache check

		if($versionstring =~ /mod_ssl/) {
		($temp1, $temp2) = split(/mod_ssl\//, $versionstring);
		($modsslversion, $temp1) = split(/\s/, $temp2);
				log_print("mod_ssl $modsslversion found\n");
    		}
	if($versionstring =~ /PHP/) {
		($temp2, $temp2) = split(/PHP\//, $versionstring);
		($phpversion, $temp1) = split(/\s/, $temp2);
		log_print("PHP $phpversion found\n");
	}	
	log_print("\n");
	undef($versionstring); undef($temp1); undef($temp2); undef($modsslversion); undef($phpversion);
}


sub isapi($) {

my $target = shift;
my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');

if($sock) {
	print $sock "GET /null.htr HTTP/1.0\r\n\r\n";
        sleep(3);
        sysread($sock, $buff, 1000);
	if($buff =~ /Error: The requested file could not be found./) {
	log_print("* HTR ISAPI filter mapped\n");
	log_print("* http://online.securityfocus.com/bid/4474/info/\n\n");
	close($sock);
	}
	}

my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');

if($sock) {
        print $sock "GET /null.printer HTTP/1.0\r\n\r\n";
        sleep(3);
        sysread($sock, $buff, 1000);
        if($buff =~ /Error in web printer install/) {
        log_print("* Printer ISAPI filter mapped\n");
        log_print("* http://online.securityfocus.com/bid/2674/info/\n\n");
        close($sock);
        }
        }

}

sub propfind($) {
my $target = shift;
my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');

if($sock) {
if($host_resolves = 1) {
print $sock "PROPFIND http://$host/ HTTP/1.1
Host:
Content-Length: 0\r\n\r\n\r\n\r\n";
} else {
print $sock "PROPFIND / HTTP/1.1
Host:
Content-Length: 0\r\n\r\n\r\n\r\n";
}
        sleep(3);
        sysread($sock, $buff, 1000);
                if($buff =~ /207 Multi-Status/) {
                log_print("* PROPFIND Information\n");
                log_print("* http://www.nextgenss.com/advisories/iisip.txt\n");
                ($temp, $return) = split(/http\:\/\//, $buff);
                ($return, $temp) = split(/\//, $return);
                ($temp, $datereturn) = split(/rfc1123\"\>/, $buff);
                ($datereturn, $temp) = split(/\<\/a/, $datereturn);
                log_print("* Server gave host/ip address: $return\n");
                log_print("* Server gave back date: $datereturn\n\n");
                undef($temp); undef($return); undef($datereturn);
                }
        }
	close($sock);
}

sub endbanner {
	log_print("--->\n");
	log_print("- All scans done. Hackbot $version -\n");
	log_print("---> Exiting.\n");
}

sub spamcheck {
	if (&check_ip($target)) {
		($a, $b, $c, $d) = split(/\./, $target);
		$spamcop = ".bl.spamcop.net";
		$spamhost = $d . "." . $c . "." . $b . "."  . $a . $spamcop;
		my $spammer = gethost($spamhost);
		if($spammer) {
		log_print("This is a known spammer\!\!, thank you spamcop \(www.spamcop.net\)\n\n");
		}
		if(!$spammer) {
		log_print("It is not a known spammer, thank you spamcop \(www.spamcop.net\)\n\n"); 
		}
	}
	undef($spammer); undef($spamhost);
}

sub proxytest {

        $proxytest = @_[0];
        $target    = @_[1];

	my $sock = new IO::Socket::INET (PeerAddr => $proxytest, PeerPort => $proxyport, Proto => 'tcp', Timeout => '10');
	if($sock) {
	log_print("Proxy responds.. now lets test if it can be used.\n\n");   
	$responds = 1;
	} else { 
		log_print("Proxy connect failed, scanning without proxy\n\n");
		undef($proxytest); undef($proxy); undef($proxyport);
	}
	if($responds) {
		print $sock "GET http://$target/_bla_\r\n\r\n";
		sleep(5);
		sysread($sock, $buff, 1000);

		if($buff =~ /404/) {
			log_print("Proxying is allowed\n\n");
		} else {
		log_print("Proxying is not allowed, will continue scan without using a proxy\n\n");
		undef($proxy); undef($proxyport); undef($proxytest); 
		}
	}
}

sub xcheck {
	log_print("\nChecking if we are allowed to connect to X\n");
	log_print("------------------------------------------\n");
	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => '6000', Proto => 'tcp', Timeout => '5');
		if($sock) {
		print $sock "\x6c\x00\x0b\x00\x00\x00\x00\x00\x00\x00\x00\x00";
		sleep(3);
		sysread($sock, $buff, 10000);
			if($buff !~ /protocol/i && $buff !~ /autho/i) {
				log_print("Allowed to connect!\n\n");
			} else {
			log_print("Connection not allowed\n\n");
 		} 
	}
	if(!$sock) {
	log_print("No X\n\n");
	}
}
